package com.zayac.changeurl.entity;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ReUtil;
import com.zayac.changeurl.annotation.MsgAnnotation;
import com.zayac.changeurl.common.Constant;
import lombok.Data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.lang.reflect.Field;
import java.util.stream.Collectors;

/**
 * @author zayac
 * @since 2023-09-08 22:54
 */
@Data
public class KyMsgEntity {
    @MsgAnnotation(startWith = "WEB")
    private List<String> pc;
    @MsgAnnotation(startWith = "H5")
    private List<String> h5;
    @MsgAnnotation(startWith = "全站")
    private List<String> app;
    @MsgAnnotation(startWith = "体育")
    private List<String> ty;
    @MsgAnnotation(startWith = "WEB")
    private String fljPc;
    @MsgAnnotation(startWith = "H5")
    private String fljH5;

    public KyMsgEntity(String[] array) {
        for (Field field : this.getClass().getDeclaredFields()) {
            // 获取字段上的MsgAnnotation注解
            MsgAnnotation msgAnnotation = field.getAnnotation(MsgAnnotation.class);
            // 如果注解不为空，说明该字段使用了MsgAnnotation注解
            if (msgAnnotation != null) {
                // 获取注解的startWith属性值
                String startWith = msgAnnotation.startWith();
                // 创建一个列表，用于存储匹配的字符串
                List<String> matched = new ArrayList<>();
                // 遍历字符串数组，查找以startWith开头的字符串
                for (String s : array) {
                    if (s.startsWith(startWith)) {
                        // 如果找到匹配的字符串，添加到列表中
                        matched.add(s);
                    }
                }

                // 设置字段的访问权限为true，以便赋值操作
                field.setAccessible(true);
                try {
                    // 判断字段的类型是否是List 如果是List类型，直接将匹配的列表赋值给该字段
                    if (field.getType().isAssignableFrom(List.class)) {
                        //优先将包含新字的放到List第一位置
                        matched = matched.stream()
                                .sorted(Comparator.comparing(str -> !str.contains(Constant.PREFER)))
                                .map(x -> ReUtil.get(Constant.URL_PATTERN, x, 0))
                                .collect(Collectors.toList());
                        //断言 list中不含有null值，防止修改数据为空
                        Assert.noNullElements(ArrayUtil.toArray(matched, String.class));
                        field.set(this, matched);
                    } else {
                        // 如果不是List类型，判断匹配的列表是否为空
                        if (!matched.isEmpty()) {
                            // 如果不为空，取第一个元素赋值给该字段
                            field.set(this, ReUtil.get(Constant.URL_PATTERN, matched.get(0), 0));
                        }
                    }
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
